#ifndef IN_EQUIS_ARRAY_HPP
#error This file must be included via "array.hpp"
#endif

EQUIS_ENTER_NS1(equis);

template < typename T > inline
array<T>::array()
    : m_ptr(NULL), m_size(0), m_free_ptr(false)
{
}

template < typename T > inline
array<T>::array(T* ptr, long size, bool free_ptr)
    : m_ptr(ptr), m_size(size), m_free_ptr(free_ptr)
{}

template < typename T > inline
array<T>::~array() {
    if (m_ptr && m_free_ptr) {
        _aligned_free(m_ptr);
    }
}

template < typename T > inline
void
array<T>::clear(const T& value) {
	for (long i = 0; i < m_size; ++i) {
		m_ptr[i] = value;
	}
}

template < typename T > inline
bool
array<T>::resize(long size, const T& value) {
    if (size == m_size) return true;

	T* new_ptr = reinterpret_cast<T*>(
		_aligned_malloc(size * sizeof(T), 16));
	if (!new_ptr) return false;

	long copy = min(size, m_size);

	// Copy preserved cells
	for (long i = 0; i < copy; ++i) {
		new (&new_ptr[i]) T(m_ptr[i]);
	}

	// Initialize remaining cells
	for (long i = copy; i < size; ++i) {
		new (&new_ptr[i]) T(value);
	}

	// Destroy old cells
	for (long i = 0; i < m_size; ++i) {
		m_ptr[i].~T();
	}

	m_ptr = new_ptr;
	m_size = size;
	m_free_ptr = true;
	return true;
}

template < typename T > inline
long
array<T>::size() const {
	return m_size;
}

template < typename T > inline
bool
array<T>::valid(long index) const {
	return index >= 0 && index < m_size;
}

template < typename T > inline
T&
array<T>::operator [] (long index) {
	EQUIS_DEBUG_ASSERT(valid(index));
	return m_ptr[index];
}

template < typename T > inline
const T&
array<T>::operator [] (long index) const {
	EQUIS_DEBUG_ASSERT(valid(index));
	return m_ptr[index];
}

template < typename T > inline
typename array<T>::iterator
array<T>::begin() {
	return &m_ptr[0];
}

template < typename T > inline
typename array<T>::iterator
array<T>::end() {
	return &m_ptr[size()];
}

template < typename T > inline
typename array<T>::const_iterator
array<T>::begin() const {
	return &m_ptr[0];
}

template < typename T > inline
typename array<T>::const_iterator
array<T>::end() const {
	return &m_ptr[size()];
}

EQUIS_LEAVE_NS1(equis);
