#ifndef EQUIS_ARRAY2_HPP
#define EQUIS_ARRAY2_HPP

EQUIS_ENTER_NS1(equis);

template < typename T >
class array2 {
	EQUIS_NO_COPY(array2);
public:
    typedef array2<T> this_type;
    typedef T value_type;
    typedef value_type* pointer;
    typedef value_type& reference;
    typedef const value_type& const_reference;
    typedef value_type* iterator;
    typedef const value_type* const_iterator;

    array2();
    array2(T* ptr, long rows, long cols, bool free_ptr = false);
    ~array2();

    void clear(const T& value = T());
    bool resize(long rows, long cols, const T& value = T());
    
	bool empty() const;
    long rows() const;
	long cols() const;
	long size() const;

    bool valid(long row, long col) const;

    T& operator [] (long index);
    const T& operator [] (long index) const;

	T& operator () (long row, long col);
    const T& operator () (long row, long col) const;

	T& at(long row, long col);
    const T& at(long row, long col) const;

    iterator begin();
    iterator end();
    const_iterator begin() const;
    const_iterator end() const;

	iterator row(long index);
	const_iterator row(long index) const;
    
private:
    T* m_ptr;
    long m_rows;
	long m_cols;
    bool m_free_ptr;
};

EQUIS_LEAVE_NS1(equis);

#define IN_EQUIS_ARRAY2_HPP
#include "array2.inl"
#undef IN_EQUIS_ARRAY2_HPP

#endif /* EQUIS_ARRAY2_HPP */
