#ifndef EQUIS_TYPES_HPP
#define EQUIS_TYPES_HPP

#if defined(HAVE_STDINT_H)
#include <stdint.h>
#endif

EQUIS_ENTER_NS1(equis);

template < typename T > struct numeric_traits;

#if defined(HAVE_STDINT_H)
typedef int8_t int8;
typedef int16_t int16;
typedef int32_t int32;
typedef int64_t int64;
typedef uint8_t uint8;
typedef uint16_t uint16;
typedef uint32_t uint32;
typedef uint64_t uint64;

#elif defined(HAVE_MS_FIXED_INT)
typedef __int8 int8;
typedef __int16 int16;
typedef __int32 int32;
typedef __int64 int64;
typedef unsigned __int8 uint8;
typedef unsigned __int16 uint16;
typedef unsigned __int32 uint32;
typedef unsigned __int64 uint64;

#else
#error Unable to define fixed-width integer types.
#endif

#if defined(HAVE_IEEE_FLOAT)
typedef float flt32;
typedef double flt64;
#endif

/// @name Byte ordering macros
//@{

#if BYTE_ORDER == 1234
#define EQUIS_NTOH2(A, B) A, B
#define EQUIS_NTOH4(A, B, C, D) A, B, C, D
#define EQUIS_NTOH2S(A, B) A; B
#define EQUIS_NTOH4S(A, B, C, D) A; B; C; D
#elif BYTE_ORDER == 4321
#define EQUIS_NTOH2(A, B) B, A
#define EQUIS_NTOH4(A, B, C, D) D, C, B, A
#define EQUIS_NTOH2S(A, B) B; A
#define EQUIS_NTOH4S(A, B, C, D) D; C; B; A
#else
#error Unable to support the native byte ordering.
#endif

#define EQUIS_NTOH8(A, B, C, D, E, F, G, H) \
EQUIS_NTOH2(EQUIS_NTOH4(A, B, C, D), EQUIS_NTOH4(E, F, G, H))

#define EQUIS_NTOH16(A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P) \
EQUIS_NTOH4(EQUIS_NTOH4(A, B, C, D), EQUIS_NTOH4(E, F, G, H), \
             EQUIS_NTOH4(I, J, K, L), EQUIS_NTOH4(M, N, O, P))

#define EQUIS_NTOH8S(A, B, C, D, E, F, G, H) \
EQUIS_NTOH2S(EQUIS_NTOH4S(A, B, C, D), EQUIS_NTOH4S(E, F, G, H))

#define EQUIS_NTOH16S(A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P) \
EQUIS_NTOH4S(EQUIS_NTOH4S(A, B, C, D), EQUIS_NTOH4S(E, F, G, H), \
              EQUIS_NTOH4S(I, J, K, L), EQUIS_NTOH4S(M, N, O, P))

//@}

/// 16-bit combination of fixed-width types.
union raw16 {
    int16 s;
    uint16 us;
    int8 EQUIS_NTOH2(b1, b0);
    uint8 EQUIS_NTOH2(ub1, ub0);
};

/// 32-bit combination of fixed-width types.
union raw32 {
    int32 i;
    uint32 ui;
    flt32 f;
    struct { int16 EQUIS_NTOH2(s1, s0); };
    struct { uint16 EQUIS_NTOH2(us1, us0); };
    struct { int8 EQUIS_NTOH4(b3, b2, b1, b0); };
    struct { uint8 EQUIS_NTOH4(ub3, ub2, ub1, ub0); };
};

/// 64-bit combination of fixed-width types.
union raw64 {
    int64 l;
    uint64 ul;
    flt64 d;
    
    struct { int16 EQUIS_NTOH4(s3, s2, s1, s0); };
    struct { uint16 EQUIS_NTOH4(us3, us2, us1, us0); };
    struct { int8 EQUIS_NTOH8(b7, b6, b5, b4, b3, b2, b1, b0); };
    struct { uint8 EQUIS_NTOH8(ub7, ub6, ub5, ub4, ub3, ub2, ub1, ub0); };
};

struct vec2f { flt32 x, y; };
struct vec3f { flt32 x, y, z; };

EQUIS_LEAVE_NS1(equis);

#define IN_EQUIS_TYPES_HPP
#include "types.inl"
#undef IN_EQUIS_TYPES_HPP

#endif /* EQUIS_TYPES_HPP */
