#ifndef IN_EQUIS_WIN32_COMPTR_HPP
#error This file must be included via ComPtr.hpp
#endif

template < typename T > inline
ComPtr<T>::ComPtr()
: m_ptr(NULL), m_noRelease(false)
{
}

template < typename T > inline
ComPtr<T>::ComPtr(const ComPtr<T>& src)
: m_ptr(NULL), m_noRelease(false)
{
    Attach(src);
}

template < typename T > inline
ComPtr<T>::ComPtr(T* ptr)
: m_ptr(NULL), m_noRelease(false)
{
    Attach(ptr);
}

template < typename T > inline
ComPtr<T>::~ComPtr()
{
    Detach();
}

template < typename T > inline bool
ComPtr<T>::IsNull() const
{
    return m_ptr == NULL;
}

template < typename T > inline bool
ComPtr<T>::IsNotNull() const
{
    return m_ptr != NULL;
}

template < typename T > inline T*
ComPtr<T>::operator -> () const
{
    _ASSERT(m_ptr != NULL);
    return m_ptr;
}

template < typename T > inline
ComPtr<T>::operator T* () const
{
    return m_ptr;
}

template < typename T > inline T*
ComPtr<T>::Get() const
{
    if (m_ptr != NULL) m_ptr->AddRef();
    return m_ptr;
}

template < typename T > inline T*
ComPtr<T>::GetWithoutAddRef() const
{
    return m_ptr;
}

template < typename T > inline ComPtr<T>&
ComPtr<T>::operator = (const ComPtr<T>& src)
{
    Attach(src);
    return *this;
}

template < typename T > inline ComPtr<T>&
ComPtr<T>::operator = (T *ptr)
{
    Attach(ptr);
    return *this;
}

template < typename T > inline void
ComPtr<T>::Attach(T *ptr)
{
    Detach();
    m_ptr = ptr;
    if (m_ptr != NULL) m_ptr->AddRef();
}

template < typename T > inline void
ComPtr<T>::AttachWithoutAddRef(T* ptr)
{
    Detach();
    m_ptr = ptr;
}

template < typename T > inline void
ComPtr<T>::AttachWithoutRelease(T* ptr)
{
    Attach(ptr);
    m_noRelease = true;
}

template < typename T > inline void
ComPtr<T>::AttachWithoutAddRefOrRelease(T* ptr)
{
    AttachWithoutAddRef(ptr);
    m_noRelease = true;
}

template < typename T > inline void
ComPtr<T>::Detach()
{
    if (m_ptr != NULL && !m_noRelease) m_ptr->Release();

    m_ptr = NULL;
    m_noRelease = false;
}

template < typename T > inline T*
ComPtr<T>::DetachWithoutRelease()
{
    T* oldPtr = m_ptr;

    m_ptr = NULL;
    m_noRelease = false;

    return oldPtr;
}

template < typename T > inline T**
ComPtr<T>::DetachAndGetPointer()
{
    Detach();
    return &m_ptr;
}

template < typename T > inline void**
ComPtr<T>::DetachAndGetVoidPointer()
{
    return reinterpret_cast<void**>(
        DetachAndGetPointer());
}
