#include "common.hpp"
EQUIS_USING_NS1(equis);

void
MediaTypePtr::Clone(AM_MEDIA_TYPE* ptr) {
	Clear();

	// Nothing more to do if the incoming pointer is NULL.
	if (!ptr) return;

	AM_MEDIA_TYPE* new_ptr = reinterpret_cast<AM_MEDIA_TYPE*>(
		CoTaskMemAlloc(sizeof(*new_ptr)));
	if (!new_ptr) {
		// No cleanup necessary.
		return;
	}

	*new_ptr = *ptr;
	if (new_ptr->pUnk) {
		new_ptr->pUnk->AddRef();
	}

	if (new_ptr->cbFormat) {
		new_ptr->pbFormat = reinterpret_cast<BYTE*>(
			CoTaskMemAlloc(new_ptr->cbFormat));
		if (!new_ptr->pbFormat) {
			// Failed to allocate format memory. Free the format and abort.
			CoTaskMemFree(new_ptr);
			return;
		}
		memcpy(new_ptr->pbFormat, ptr->pbFormat, new_ptr->cbFormat);
	}
	else {
		new_ptr->pbFormat = NULL;
	}

	m_ptr = new_ptr;
}

void
MediaTypePtr::Clear() {
	if (m_ptr && !m_noRelease) {
		if (m_ptr->pUnk) m_ptr->pUnk->Release();
		if (m_ptr->pbFormat) CoTaskMemFree(m_ptr->pbFormat);
		CoTaskMemFree(m_ptr);
	}
	m_ptr = NULL;
	m_noRelease = false;
}

