using System;
using System.Collections.Generic;
using System.Text;
using Equis.Xna.Input;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Demos.Rocks {
    public class Background : Shape {
        private readonly VertexPositionTexture[] planeVertices = {
            new VertexPositionTexture(new Vector3(-2000.0f, -2000.0f, -10000.0f), new Vector2(0.0f, 1.0f)),
            new VertexPositionTexture(new Vector3(-2000.0f, +2000.0f, -10000.0f), new Vector2(0.0f, 0.0f)),
            new VertexPositionTexture(new Vector3(+2000.0f, +2000.0f, -10000.0f), new Vector2(1.0f, 0.0f)),
            new VertexPositionTexture(new Vector3(+2000.0f, -2000.0f, -10000.0f), new Vector2(1.0f, 1.0f))
        };

        private BasicEffect effect;
        private VertexBuffer vertexBuffer;
        private VertexDeclaration vertexDeclaration;

        public Background(RocksGame game)
            : base(game) {
        }

        public Texture2D VideoFrame {
            get;
            set;
        }

        public override void Setup() {
            GraphicsDevice graphics = Game.GraphicsManager.GraphicsDevice;

            effect = new BasicEffect(graphics, null);

            vertexBuffer = new VertexBuffer(graphics,
                VertexPositionTexture.SizeInBytes * planeVertices.Length,
                BufferUsage.None);

            vertexDeclaration = new VertexDeclaration(graphics, VertexPositionTexture.VertexElements);

            base.Setup();
        }

        public override void Update(GameTime gameTime) {
            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime) {
            effect.Projection = Game.Projection;
            effect.View = Game.View;
            effect.World = Matrix.Identity;

            effect.TextureEnabled = true;
            if (VideoFrame != null) {
                effect.Texture = VideoFrame;
            } else if (Texture != null) {
                effect.Texture = Texture;
            } else {
                effect.Texture = null;
            }

            GraphicsDevice graphics = Game.GraphicsManager.GraphicsDevice;

            { // Update vertex coordinates
                float aspectRatio = (float)graphics.Viewport.Width / (float)graphics.Viewport.Height;

                for (int i = 0; i < planeVertices.Length; ++i) {
                    int xSign = Math.Sign(planeVertices[i].Position.X);
                    int ySign = Math.Sign(planeVertices[i].Position.Y);

                    planeVertices[i].Position.X = xSign * 10000;
                    planeVertices[i].Position.Y = ySign * 10000 / aspectRatio;
                }

                // TODO add modification of the texture coordinates to match the calibration.
            }

            effect.Begin(SaveStateMode.SaveState);

            graphics.VertexDeclaration = vertexDeclaration;

            vertexBuffer.SetData(planeVertices);
            graphics.Vertices[0].SetSource(vertexBuffer, 0, VertexPositionTexture.SizeInBytes);

            foreach (EffectPass pass in effect.CurrentTechnique.Passes) {
                pass.Begin();

                graphics.DrawPrimitives(PrimitiveType.TriangleFan, 0, planeVertices.Length);

                pass.End();
            }
            effect.End();

            base.Draw(gameTime);
        }
    }
}
