#include "common.hpp"
#include "demo-common/View.hpp"
#include "touch/touch.h"
#include "touch/calibration.hpp"
#include "video/input.h"
EQUIS_USING_NS1(equis);

static ETSurface* s_touch_surface;
static array2<uint32> s_preview;

static ETEvent s_events[64];
static long s_events_end;

void DisplayPaint(HWND window, HDC dc, BOOL erase, RECT region) {

	if (!s_preview.empty()) {
		BITMAPINFO bitmap_info = { {
			sizeof(bitmap_info),
			s_preview.cols(), -(long)s_preview.rows(), 1,
			32, BI_RGB
		} };

		if (!SetDIBitsToDevice(
			dc, 0, 0, s_preview.cols(), s_preview.rows(),
			0, 0, 0, s_preview.rows(),
			s_preview.begin(), &bitmap_info, DIB_RGB_COLORS))
		{
			ReportError(window, _T("Failed to update display: %s"), GetLastErrorString());
		}

#if 0
		SetDCBrushColor(dc, RGB(0xFF, 0x00, 0xFF));
		SetDCPenColor(dc, RGB(0xFF, 0x00, 0xFF));
		for (long x = 32; x < s_preview.cols(); x += 64) {
			for (long y = 32; y < s_preview.rows(); y += 64) {
				Ellipse(dc, x - 4, y - 4, x + 4, y + 4);
			}
		}
#endif
	}
}

void DisplayIdle(HWND window) {

	s_events_end = etPoll(s_touch_surface, s_events, EQUIS_ARRAY_CAPACITY(s_events));

	ETEvent* events = s_events;
	for (long i = 0; i < s_events_end; ++i) {
		const char* op;
		if (events[i].type > 0) op = "add";
		else if (events[i].type < 0) op = "del";
		else op = "set";

		fprintf(stdout, "%ld %s %ld %ld",
			events[i].frame, op, events[i].cursor_id, events[i].session_id);
		if (events[i].type >= 0) {
			fprintf(stdout, " %f %f", events[i].x, events[i].y);
		}
		fputc('\n', stdout);
	}

	EVFormat format;
	if (etGetFormat(s_touch_surface, &format) == 0) {
		if (!s_preview.resize(format.height, format.width)) return;

		// Preview is (currently) always RGB32.
		etPreview(s_touch_surface, ET_PREVIEW_FILTERED,
			s_preview.begin(), 4 * s_preview.cols(), 4 * s_preview.size());
		RedrawWindow(window, NULL, NULL, RDW_INVALIDATE|RDW_VALIDATE|RDW_UPDATENOW);
	}
}

int main(void) {
	CoInitialize(NULL);

	s_touch_surface = etCreate();

	etSetf(s_touch_surface, ET_PIXEL_THRESHOLD, 0.1f);
	etSetf(s_touch_surface, ET_TOUCH_THRESHOLD, 5.0f);

	etSourceVideoOpen(s_touch_surface, "dshow:1");

	HWND window = CreateDisplayWindow(GetModuleHandle(NULL), 640, 480);// format.width, format.height);
	ShowWindow(window, SW_NORMAL);
	return RunMessageLoop(window);
}
