#ifndef EQUIS_IMAGE_BLOB_DETECT_HH
#define EQUIS_IMAGE_BLOB_DETECT_HH

EQUIS_USING_NS1(equis);

#include "stats.hpp"
#include "blob.hpp"

//EQUIS_ENTER_NS2(equis,image);

class BlobDetector {
public:
    BlobDetector();

    uint16 GetThreshold() const;
    void SetThreshold(uint16 value);

    float GetMinStrength() const;
    void SetMinStrength(float value);
    
    void Update(const array2<uint16>& img);
    long Peek(vector<Blob> &values);
    
    FrameStats& Stats() { return m_stats; }

private:
    uint16 m_threshold;
    float m_min_strength;

    FrameStats m_stats;

    struct BlobI : public Blob {
        /** Merge-tree parent of this blob. Undefined in m_best. */
        BlobI* parent;

        /** Row to which refs applies. */
        long row;

        /** Number of references to thie blob in 'row'. */
        long refs;
    };
    
    /** Preallocated blobs. */
    array<BlobI> m_blobs;

    /** Top of the free blob stack. Each parent pointer indicates the
        next blob in the stack. */
    BlobI* m_free;
    
    /** Blob of each pixel in the current row. NULL if the pixel does
        not exceed threshold. */
    vector<BlobI*> m_row;

    /** List of highest-magnitude blobs. This is used as a reversed
        heap during updates. */
    vector<Blob> m_best;

    void ClearState(long rows, long cols);
    void HeapBubbleUp();

    static BlobI* GetRoot(BlobI* blob);
    static bool IsAncestor(BlobI* root, BlobI* child);
};

//EQUIS_LEAVE_NS2(equis,image);

#endif /* EQUIS_IMAGE_BLOB_DETECT_HH */
