#include "common.hpp"
#include "ArrayEnumMediaTypes.hpp"
#include "MediaType.hpp"
EQUIS_USING_NS1(equis);

ArrayEnumMediaTypes::ArrayEnumMediaTypes(AM_MEDIA_TYPE* types, ULONG length)
: m_types(types), m_length(length), m_index(0)
{}

ArrayEnumMediaTypes::~ArrayEnumMediaTypes()
{
}

STDMETHODIMP ArrayEnumMediaTypes::QueryInterface(REFIID iid, void** result) {
    HRESULT res = BaseUnknown::QueryInterface(iid, result);
    if (res == E_NOINTERFACE) {
        if (IsEqualIID(iid, IID_IEnumMediaTypes)) {
            *result = implicit_cast<IEnumMediaTypes*>(this);
        }
        if (*result) {
            AddRef();
            res = S_OK;
        }
    }
    return res;
}

STDMETHODIMP ArrayEnumMediaTypes::Next(ULONG length, AM_MEDIA_TYPE **types, ULONG *used)
{
    HRESULT res = S_OK;

    if (types == NULL || (length != 1 && used == NULL)) {
        res = E_POINTER;
    } else {
        ULONG position;
        for (position = 0; position < length && m_index + position < m_length; ++position)
        {
            res = CloneMediaType(&types[position], &m_types[m_index + position]);
            if (FAILED(res)) break;
        }

        m_index += position;
        if (used != NULL) *used = position;
        if (res == S_OK && position < length) res = S_FALSE;
    }

    return res;
}

STDMETHODIMP ArrayEnumMediaTypes::Skip(ULONG length)
{
    HRESULT res = S_OK;

    {
        size_t position = std::max(m_length - m_index, length);

        m_index += position;
        if (res == S_OK && position < length) res = S_FALSE;
    }

    return res;
}

STDMETHODIMP ArrayEnumMediaTypes::Reset()
{
    HRESULT res = S_OK;

    {
        m_index = 0;
    }

    return res;
}

STDMETHODIMP ArrayEnumMediaTypes::Clone(IEnumMediaTypes **types)
{
    HRESULT res = S_OK;

    if (types == NULL) {
        res = E_POINTER;
    } else {
        *types = new ArrayEnumMediaTypes(m_types, m_length);
    }

    return res;
}
