#pragma once

#include "MediaType.hpp"

class BaseSink : public equis::BaseUnknown, public IBaseFilter, public IPin, public IMemInputPin {
    EQUIS_NO_COPY(BaseSink)
    BASEUNKNOWN_DECLARE(BaseUnknown)

public:
    const AM_MEDIA_TYPE* GetConnectionType() const;
    STDMETHOD(ChangingConnectionType)(const AM_MEDIA_TYPE* type) = 0;

    // IPersist
    STDMETHOD(GetClassID)(CLSID *clsid);

    // IMediaFilter
    STDMETHOD(Stop)(VOID);
    STDMETHOD(Pause)(VOID);
    STDMETHOD(Run)(REFERENCE_TIME startTime);
    STDMETHOD(GetState)(DWORD timeout, FILTER_STATE *result);
    STDMETHOD(SetSyncSource)(IReferenceClock *clock);
    STDMETHOD(GetSyncSource)(IReferenceClock **clock);

    // IFilterBase
    STDMETHOD(EnumPins)(IEnumPins **enumPins);
    STDMETHOD(FindPin)(LPCWSTR id, IPin **pin);
    STDMETHOD(JoinFilterGraph)(IFilterGraph *graph, LPCWSTR name);
    STDMETHOD(QueryFilterInfo)(FILTER_INFO *info);
    STDMETHOD(QueryVendorInfo)(LPWSTR *vendorInfo);

    // IPin
    STDMETHOD(Connect)(IPin *targetPin, const AM_MEDIA_TYPE *type);
    STDMETHOD(ReceiveConnection)(IPin *sourcePin, const AM_MEDIA_TYPE *type);
    STDMETHOD(Disconnect)(VOID);
    STDMETHOD(ConnectedTo)(IPin **pin);
    STDMETHOD(ConnectionMediaType)(AM_MEDIA_TYPE *type);
    STDMETHOD(QueryPinInfo)(PIN_INFO *pinInfo);
    STDMETHOD(QueryDirection)(PIN_DIRECTION *pinDirection);
    STDMETHOD(QueryId)(LPWSTR *id);
    STDMETHOD(QueryAccept)(const AM_MEDIA_TYPE *type) = 0;
    STDMETHOD(EnumMediaTypes)(IEnumMediaTypes **types) = 0;
    STDMETHOD(QueryInternalConnections)(IPin **pins, ULONG *length);
    STDMETHOD(EndOfStream)(VOID);
    STDMETHOD(BeginFlush)(VOID);
    STDMETHOD(EndFlush)(VOID);
    STDMETHOD(NewSegment)(REFERENCE_TIME startTime, REFERENCE_TIME stopTime, double rate);

    // IMemInputPin
    STDMETHOD(GetAllocator)(IMemAllocator **allocator);
    STDMETHOD(NotifyAllocator)(IMemAllocator *allocator, BOOL readOnly);
    STDMETHOD(GetAllocatorRequirements)(ALLOCATOR_PROPERTIES *properties);

    STDMETHOD(Receive)(IMediaSample *sample) = 0;
    STDMETHOD(ReceiveMultiple)(IMediaSample **samples, LONG capacity, LONG *length);
    STDMETHOD(ReceiveCanBlock)(VOID);

protected:
    BaseSink(HRESULT& res);
    virtual ~BaseSink();

	equis::Mutex m_lock;

private:
    // Pin state
    ComPtr<IPin> m_sourcePin;
    MediaType m_connectionMediaType;

    // Filter state
    REFERENCE_TIME m_startTime;
    FILTER_STATE m_state;
    ComPtr<IReferenceClock> m_clock;
    ComPtr<IFilterGraph> m_graph;
    WCHAR m_name[MAX_PIN_NAME];
    IPin *m_pins[1];

    bool m_flushing;
};

#define IN_DSHOW_VIDEOSINK_HPP
#include "BaseSink.inl"
#undef IN_DSHOW_VIDEOSINK_HPP
