#ifndef EQUIS_VIDEO_INPUT_SPI_H
#define EQUIS_VIDEO_INPUT_SPI_H

#include "input.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct EVInputSpi EVInputSpi;

struct EVInput {
	const EVInputSpi* spi;
};

struct EVInputFrame {
	EVInput* origin;
};

struct EVInputSpi {
	int (*Enum)(EVInputSpi* self, int start, char* uris, int uris_cap);
	EVInput* (*Open)(EVInputSpi* self, const char* uri);
	EVInput* (*OpenFile)(EVInputSpi* self, const char* path);

	int (*Close)(EVInput* self);
	int (*GetFormats)(EVInput* self, int start, EVFormat* formats, int formats_cap);
	int (*SetFormat)(EVInput* self, EVFormat* format);
	int (*GetFormat)(EVInput* self, EVFormat* format);
	EVInputFrame* (*Poll)(EVInput* self);
	EVInputFrame* (*Next)(EVInput* self);
	int (*Callback)(EVInput* self, EVICallbackProc callback, void* user);

	const EVFormat* (*FrameFormat)(EVInputFrame* frame);
	long (*FrameSize)(EVInputFrame* frame);
	void* (*FramePointer)(EVInputFrame* frame);
	int (*FrameDone)(EVInputFrame* frame);

	int (*Geti)(EVInput* self, int name, int* value);
	int (*Getf)(EVInput* self, int name, float* value);
	int (*Seti)(EVInput* self, int name, int value);
	int (*Setf)(EVInput* self, int name, float value);
	int (*GetCapsf)(EVInput* self, int name, float* min, float* max, float* step, float* def);
	int (*GetStopsf)(EVInput* self, int name, float* stops, int stops_cap);

	// Used by eviRegisterSpi to maintain a stack of SPIs.
	EVInputSpi* next;
};

void eviRegisterSpi(EVInputSpi* spi);

#ifdef __cplusplus
}
#endif

#endif /* EQUIS_VIDEO_INPUT_SPI_H */

