#include "common.hpp"
#include "xna-common.hpp"
#include "Input_Video.hpp"

EQUIS_USING_NS3(Equis,Xna,Input);

static HRESULT GetComPtr(Object ^managed, REFIID iid, void **result)
{
	FieldInfo ^field = managed->GetType()->GetField("pComPtr", BindingFlags::Instance | BindingFlags::NonPublic);
	IUnknown *unknown = (IUnknown *) Pointer::Unbox(field->GetValue(managed));
	return unknown->QueryInterface(iid, result);
}

#if 0
Video ^Video::FromCapture(VideoDevice ^captureDevice)
{
	return FromCapture(captureDevice->Path);
}
#endif

Video ^Video::FromCapture(String ^capturePath)
{
	IntPtr capturePathUni;
	EVInput *video = NULL;
	try {
		capturePathUni = Marshal::StringToCoTaskMemAnsi(capturePath);
		
		video = eviOpen(reinterpret_cast<char*>(capturePathUni.ToPointer()));
		if (!video) Marshal::ThrowExceptionForHR(E_FAIL);
	
		return gcnew Video(video);
	}
	finally {
		if (capturePathUni != IntPtr::Zero) Marshal::FreeCoTaskMem(capturePathUni);
	}
}

Video ^Video::FromFile(String ^filename)
{
	throw gcnew NotImplementedException("Video input from file is not yet supported");
}

VideoFormat ^Video::Format::get()
{
	HRESULT res;

	EVFormat nativeFormat;

	res = eviGetFormat(m_source, &nativeFormat);
	Marshal::ThrowExceptionForHR(res);

	return VideoFormat::FromNative(&nativeFormat);
}

bool Video::NextFrame(Texture2D ^texture)
{
	EVInputFrame* frame = eviPoll(m_source);
	if (!frame) return false;

	HRESULT res = S_OK;
	ComPtr<IDirect3DSurface9> texturePtr;

	res = GetComPtr(texture, texturePtr.GetUuid(), texturePtr.DetachAndGetVoidPointer());
	Marshal::ThrowExceptionForHR(res);

	return true;
}

Video::Video(EVInput *source)
: m_source(source)
{
}

Video::~Video(void)
{
	eviClose(m_source);
}
